package com.loafle.commons.rpc.registry.pojo;

import java.lang.reflect.Type;

import com.loafle.commons.rpc.RPCException;
import com.loafle.commons.rpc.protocol.RPCRegistryCodec;
import com.loafle.commons.rpc.registry.RPCRegistry;
import com.loafle.commons.rpc.util.RPCServiceRegistry;

/**
 * POJORPCRegistry
 */
public class POJORPCRegistry implements RPCRegistry {
	private RPCServiceRegistry serviceRegistry;

	public POJORPCRegistry() {
		this.serviceRegistry = new RPCServiceRegistry();
	}

	@Override
	public void registerService(Object receiver) throws RPCException {
    if (null == receiver) {
      throw new RPCException("receiver is null");
    }
		this.registerService(receiver, receiver.getClass());
	}

	@Override
	public void registerService(Object receiver, String name) throws RPCException {
    if (null == receiver) {
      throw new RPCException("receiver is null");
    }
    this.registerService(receiver, receiver.getClass(), name);
	}

	@Override
	public void registerService(Object receiver, Class<?> receiverClass) throws RPCException {
    if (null == receiver) {
      throw new RPCException("receiver is null");
    }
    this.registerService(receiver, receiverClass, receiverClass.getName());
	}

	@Override
	public void registerService(Object receiver, Class<?> receiverClass, String name) throws RPCException {
		this.serviceRegistry.register(receiver, receiverClass, name);
	}

	@Override
	public boolean hasMethod(String method) {
		return this.serviceRegistry.hasMethod(method);
	}

	@Override
	public Object invoke(RPCRegistryCodec codec) throws RPCException {
		String method = codec.method();
		if (!this.serviceRegistry.hasMethod(method)) {
			throw new RPCException(String.format("method[%s] is not exist", method));
		}
		Type[] parameterTypes = this.serviceRegistry.getParameterTypes(method);
		Object[] params = codec.params(parameterTypes);

		Object result = this.serviceRegistry.invoke(method, params);

		return result;
	}
  
}